@@ -66,7 +66,8 @@
 - (NJBTransactionResult *)changeTrackTagTo:(Track *)newTrack from:(Track *)oldTrack;
 - (NJBTransactionResult *)downloadFile:(DataFile *)dataFile;
 - (NJBTransactionResult *)uploadFile:(DataFile *)dataFile toFolder:(NSString *)path;
-- (NJBTransactionResult *)deleteFile:(NSNumber *)fileID;
+- (NJBTransactionResult *)deleteFile:(MyItem *)file fromDir:(Directory *)parentDir;
+- (NJBTransactionResult *)deleteFile:(MyItem *)file;
 - (unsigned long long)totalDiskSpace;
 - (unsigned long long)freeDiskSpace;
 - (NSCalendarDate *)jukeboxTime;
@@ -97,6 +98,7 @@
 - (uint32_t)videoAssoc;
 - (uint32_t)zencastAssoc;
 - (BOOL)isAudioType:(LIBMTP_filetype_t) filetype;
+- (BOOL)isMTPDevice;
 
 int progress (u_int64_t sent, u_int64_t total, const char* buf, unsigned len, void *data);
 int mtp_progress (uint64_t const sent, uint64_t const total, void const * const data);
