@@ -5,8 +5,6 @@
 //  Created by Richard Low on Wed Jul 21 2004.
 //
 
-// todo: can we find out how many tracks there are before we download for progress bar & set capacity in NSMutableArray?
-
 /* An obj-c wrapper for libnjb
  * most methods return an NJBTransactionResult
  * to indicate success/failure
@@ -40,6 +38,7 @@ StatusDisplayer *statusDisplayerGlobal;
 - (NSString *)rateString:(double) rate;
 - (NSString *)uniqueFilename:(NSString *)path;
 - (LIBMTP_track_t *)libmtpTrack_tFromTrack:(Track *)track;
+- (void)libmtpFolder_tToDir:(LIBMTP_folder_t *)folders withDir:(Directory *)baseDir;
 // we make this private so that the user never changes the setting directly
 // so we only change it before a transfer, so it is not changed during a transfer
 // which could mess things up
@@ -727,29 +726,39 @@ StatusDisplayer *statusDisplayerGlobal;
 	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 }
 
-- (NJBTransactionResult *)deleteFile:(NSNumber *)fileID
+- (NJBTransactionResult *)deleteFile:(MyItem *)file fromDir:(Directory *)parentDir
 {
 	if (![self isConnected])
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
 	
 	if (!mtpDevice)
 	{
-		if (NJB_Delete_Datafile(njb, [fileID unsignedIntValue]) == -1)
+		// we can't delete items with 0 ID: these are directories on libnjb devices that
+		// haven't been explicitly created
+		if ([file itemID] != 0)
 		{
-			NSString *error = [self njbErrorString];
-			NSLog(error);
-			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
+			if (NJB_Delete_Datafile(njb, [file itemID]) == -1)
+			{
+				NSString *error = [self njbErrorString];
+				NSLog(error);
+				return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
+			}
 		}
 	}
 	else
 	{
-		if (LIBMTP_Delete_Object(device, [fileID unsignedIntValue]) != 0)
+		if (LIBMTP_Delete_Object(device, [file itemID]) != 0)
 		{
 			NSString *error = @"error deleting file";
 			NSLog(error);
 			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 		}
 	}
+	// update cache
+	if (parentDir == nil)
+		NSLog(@"Cache not being updated since parentDir nil when deleting file %@", file);
+	else
+		[parentDir removeItem:file];
 	
 	[self updateDiskSpace];
 	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
@@ -881,13 +890,7 @@ StatusDisplayer *statusDisplayerGlobal;
 				NSString *folder = [NSString stringWithCString:filetag->folder];
 				// this has \\ instead of /
 				pathArray = [folder componentsSeparatedByString:@"\\"];
-				// the / on the beginning and end gives us extra empty objects
-				NSMutableArray *newPathArray = [NSMutableArray arrayWithArray:pathArray];
-				if ([[newPathArray objectAtIndex:[newPathArray count] - 1] length] == 0)
-					[newPathArray removeLastObject];
-				if ([[newPathArray objectAtIndex:0] length] == 0)
-					[newPathArray removeObjectAtIndex:0];
-				pathArray = newPathArray;
+				pathArray = [Directory normalizePathArray:pathArray];
 			}
 
 			if ([filename isEqualToString:@"."])
@@ -928,6 +931,13 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
+		// get folders first
+		LIBMTP_folder_t *folders = LIBMTP_Get_Folder_List(device);
+		[self libmtpFolder_tToDir:folders withDir:baseDir];
+		LIBMTP_destroy_folder_t(folders);
+		
+		NSDictionary *dirDict = [baseDir flatten];
+		
 		LIBMTP_file_t *files;
 		
 		files = LIBMTP_Get_Filelisting(device);
@@ -941,8 +951,6 @@ StatusDisplayer *statusDisplayerGlobal;
 		file = files;
 		while (file != NULL)
 		{
-			// TODO: use Get_Folder_List for folders first; then get files. we will only get files here
-			
 			if (![self isAudioType:file->filetype])
 			{
 				DataFile *dataFile = [[DataFile alloc] init];
@@ -951,7 +959,10 @@ StatusDisplayer *statusDisplayerGlobal;
 				[dataFile setSize:file->filesize];
 				[dataFile setItemID:file->item_id];
 				
-				[baseDir addItem:dataFile toDir:nil];
+				Directory *dir = [dirDict objectForKey:[NSNumber numberWithUnsignedInt:file->parent_id]];
+				if (dir == nil)
+					dir = baseDir;
+				[dir addItem:dataFile toDir:nil];
 				
 				[dataFile release];
 			}
@@ -1009,15 +1020,52 @@ StatusDisplayer *statusDisplayerGlobal;
 {
 	if (![self isConnected])
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
-	
+		
 	NSDate *date = [NSDate date];
 	[self setStatus:STATUS_UPLOADING_FILE];
-
+	
+	NSArray *pathArray = [path componentsSeparatedByString:@"\\"];
+	pathArray = [Directory normalizePathArray:pathArray];
+	
+	// we need to make directories if they don't exist
+		
+	NSMutableArray *destDirPathComponentsSoFar = [[NSMutableArray alloc] initWithCapacity:[pathArray count]];
+	int i = 0;
+	Directory *parentDir = cachedDataFileList;
+	for (i = 0; i < [pathArray count]; i++)
+	{
+		[destDirPathComponentsSoFar addObject:[pathArray objectAtIndex:i]];
+		Directory *dir = [cachedDataFileList itemWithPath:destDirPathComponentsSoFar];
+		if (dir == nil)
+		{
+			Directory *newDir = [[Directory alloc] initWithName:[pathArray objectAtIndex:i]];
+			
+			// we need to get just parent dir; destDirPathComponentsSoFar includes us too
+			NSMutableArray *parentDirComponents = [[NSMutableArray alloc] initWithArray:destDirPathComponentsSoFar];
+			// must have at least one item in the array to be here so is safe
+			[parentDirComponents removeLastObject];
+			NJBTransactionResult *res = [self createFolder:newDir inDir:[Directory stringFromPathArray:parentDirComponents]];
+			[parentDirComponents release];
+			parentDir = newDir;
+			// release here - is retained in createFolder when added to cache
+			[newDir release];
+			if (![res success])
+			{
+				return [[[NJBTransactionResult alloc] initWithSuccess:NO
+																								 resultString:[NSString stringWithFormat:
+																									 @"Could not create containing directory %@", 
+																									 [Directory stringFromPathArray:destDirPathComponentsSoFar]]] autorelease];
+			}
+		}
+		else
+			parentDir = dir;
+	}
+	
 	if (!mtpDevice)
 	{
 		// set turbo on/off
 		[self enableTurbo];
-	
+		
 		unsigned int fileid;
 		if (NJB_Send_File(njb, [[dataFile fullPath] fileSystemRepresentation], [[[dataFile fullPath] lastPathComponent] UTF8String], [path UTF8String], progress, NULL, &fileid) == -1 ) {
 			NSString *error = [self njbErrorString];
@@ -1030,71 +1078,66 @@ StatusDisplayer *statusDisplayerGlobal;
 	else
 	{
 		uint16_t objectFormat = LIBMTP_FILETYPE_UNKNOWN;
+		uint32_t parenthandle = [parentDir itemID];
 		
-		uint32_t parenthandle = 0;
-		
-		// a test to see how the player works with video & images. We really should
-		// implement the full directory structure for MTP and just upload to the folder
-		// we've been told to here.
+		// todo: add more object formats
 		NSString *extension = [[[dataFile fullPath] pathExtension] lowercaseString];
 		if ([extension isEqualToString:@"wmv"])
 		{
 			objectFormat = LIBMTP_FILETYPE_WMV;
-			parenthandle = [self videoAssoc];
+			//	parenthandle = [self videoAssoc];
 		}
 		else if ([extension isEqualToString:@"avi"])
 		{
 			objectFormat = LIBMTP_FILETYPE_AVI;
-			parenthandle = [self videoAssoc];
+			//	parenthandle = [self videoAssoc];
 		}
 		else if ([extension isEqualToString:@"mpg"] || [extension isEqualToString:@"mpeg"])
 		{
 			objectFormat = LIBMTP_FILETYPE_MPEG;
-			parenthandle = [self videoAssoc];
+			//	parenthandle = [self videoAssoc];
 		}
 		else if ([extension isEqualToString:@"asf"])
 		{
 			objectFormat = LIBMTP_FILETYPE_ASF;
-			parenthandle = [self videoAssoc];
+			//	parenthandle = [self videoAssoc];
 		}
 		else if ([extension isEqualToString:@"qt"])
 		{
 			objectFormat = LIBMTP_FILETYPE_QT;
-			parenthandle = [self videoAssoc];
+			//	parenthandle = [self videoAssoc];
 		}
 		else if ([extension isEqualToString:@"jpg"] || [extension isEqualToString:@"jpeg"])
 		{
 			objectFormat = LIBMTP_FILETYPE_JPEG;
-			parenthandle = [self picturesAssoc];
+			//	parenthandle = [self picturesAssoc];
 		}
 		else if ([extension isEqualToString:@"tif"] || [extension isEqualToString:@"tiff"])
 		{
 			objectFormat = LIBMTP_FILETYPE_TIFF;
-			parenthandle = [self picturesAssoc];
+			//	parenthandle = [self picturesAssoc];
 		}
 		else if ([extension isEqualToString:@"bmp"])
 		{
 			objectFormat = LIBMTP_FILETYPE_BMP;
-			parenthandle = [self picturesAssoc];
+			//	parenthandle = [self picturesAssoc];
 		}
 		else if ([extension isEqualToString:@"gif"])
 		{
 			objectFormat = LIBMTP_FILETYPE_GIF;
-			parenthandle = [self picturesAssoc];
+			//	parenthandle = [self picturesAssoc];
 		}
 		else if ([extension isEqualToString:@"pict"])
 		{
 			objectFormat = LIBMTP_FILETYPE_PICT;
-			parenthandle = [self picturesAssoc];
+			//	parenthandle = [self picturesAssoc];
 		}
 		else if ([extension isEqualToString:@"png"])
 		{
 			objectFormat = LIBMTP_FILETYPE_PNG;
-			parenthandle = [self picturesAssoc];
+			//	parenthandle = [self picturesAssoc];
 		}
 		
-		NSLog(@"uploading file with object format 0x%.02x", objectFormat);
-		
 		LIBMTP_file_t *genfile = LIBMTP_new_file_t();
 		genfile->filesize = [dataFile size];
 		genfile->filetype = objectFormat;
@@ -1109,11 +1152,13 @@ StatusDisplayer *statusDisplayerGlobal;
 			LIBMTP_destroy_file_t(genfile);
 			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 		}
-
+		
 		[dataFile setItemID:genfile->item_id];
 		LIBMTP_destroy_file_t(genfile);
 	}
-	
+	// update the cache
+	[parentDir addItem:dataFile];
+		
 	[self updateDiskSpace];
 		
 	double time = -[date timeIntervalSinceNow];
@@ -1558,7 +1603,7 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
 {
 	if (cachedDataFileList && !downloadingFiles)
 	{
-		return [cachedDataFileList copy];
+		return [[cachedDataFileList copy] autorelease];
 	}
 	else
 		return nil;
@@ -1580,17 +1625,34 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
 
 - (NJBTransactionResult *)createFolder:(Directory *)dir inDir:(NSString *)path
 {
-	if (mtpDevice)
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+	NSArray *pathArray = [path componentsSeparatedByString:@"\\"];
+	pathArray = [Directory normalizePathArray:pathArray];
+	Directory *parentDir = [cachedDataFileList itemWithPath:pathArray];
 	
-	unsigned int folderID;
-	if (NJB_Create_Folder(njb, [[NSString stringWithFormat:@"%@%@", path, [dir name]] UTF8String], &folderID) == -1)
+	if (!mtpDevice)
 	{
-		NSString *error = [self njbErrorString];
-		NSLog(error);
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
+		unsigned int folderID;
+		if (NJB_Create_Folder(njb, [[NSString stringWithFormat:@"%@%@", path, [dir name]] UTF8String], &folderID) == -1)
+		{
+			NSString *error = [self njbErrorString];
+			NSLog(error);
+			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
+		}
+		[dir setItemID:folderID];
 	}
-	[dir setItemID:folderID];
+	else
+	{
+		unsigned newDirID = LIBMTP_Create_Folder(device, [[dir name] UTF8String], [parentDir itemID]);
+		if (newDirID == 0)
+		{
+			return [[[NJBTransactionResult alloc] initWithSuccess:NO 
+																							 resultString:[NSString stringWithFormat:@"Could not create folder with parent ID 0x%04.", [parentDir itemID]]] autorelease];
+		}
+		[dir setItemID:newDirID];
+	}
+	// update cache
+	[parentDir addItem:dir];
+	
 	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 }
 
@@ -1757,9 +1819,22 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
 			break;
 	}
 		
-	libmtp_track->title = strdup([[track title] UTF8String]);
-	libmtp_track->album = strdup([[track album] UTF8String]);
-	libmtp_track->artist = strdup([[track artist] UTF8String]);
+	// we need to have title, album and artist set else some players get confused
+	if ([[track title] length] == 0)
+		libmtp_track->title = strdup("Unknown title");
+	else
+		libmtp_track->title = strdup([[track title] UTF8String]);
+	
+	if ([[track album] length] == 0)
+		libmtp_track->album = strdup("Unknown album");
+	else
+		libmtp_track->album = strdup([[track album] UTF8String]);
+	
+	if ([[track artist] length] == 0)
+		libmtp_track->artist = strdup("Unknown artist");
+	else
+		libmtp_track->artist = strdup([[track artist] UTF8String]);
+	
 	libmtp_track->genre = strdup([[track genre] UTF8String]);
 	NSString *nsTimeString = [NSString stringWithFormat:@"%.04d0101T000000.0", [track year]];
 	libmtp_track->date = strdup([nsTimeString UTF8String]);
@@ -1772,4 +1847,26 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
 	return libmtp_track;
 }
 
+- (void)libmtpFolder_tToDir:(LIBMTP_folder_t *)folders withDir:(Directory *)baseDir
+{
+	if (folders == NULL)
+		return;
+	
+	NSString *name = @"(unnamed directory)";
+	if (folders->name != NULL)
+		name = [NSString stringWithUTF8String:folders->name];
+
+	Directory *thisDir = [baseDir addNewDirectory:[NSArray arrayWithObjects:name, nil]];
+	[thisDir setItemID:folders->folder_id];
+	
+	[self libmtpFolder_tToDir:folders->sibling withDir:baseDir];
+	
+	[self libmtpFolder_tToDir:folders->child withDir:thisDir];
+}
+
+- (BOOL)isMTPDevice
+{
+	return mtpDevice;
+}
+
 @end
