@@ -70,6 +70,8 @@ static const LIBMTP_device_entry_t mtp_device_table[] = {
   { "Creative Zen MicroPhoto", 0x041e, 0x413c, DEVICE_FLAG_NONE },
   { "Creative Zen Sleek Photo", 0x041e, 0x413d, DEVICE_FLAG_NONE },
   { "Creative Zen Vision:M", 0x041e, 0x413e, DEVICE_FLAG_NONE },
+  // Reported by marazm@o2.pl
+  { "Creative Zen V", 0x041e, 0x4150, DEVICE_FLAG_NONE },
 
   /*
    * Samsung
@@ -132,6 +134,8 @@ static const LIBMTP_device_entry_t mtp_device_table[] = {
   { "iRiver T10", 0x4102, 0x1117, DEVICE_FLAG_NONE },
   { "iRiver T20", 0x4102, 0x1118, DEVICE_FLAG_NONE },
   { "iRiver T30", 0x4102, 0x1119, DEVICE_FLAG_NONE },
+  // Reported by David Wolpoff
+  { "iRiver T10 2GB", 0x4102, 0x1120, DEVICE_FLAG_NONE },
   // Reported by Adam Torgerson
   { "iRiver Clix", 0x4102, 0x112a, DEVICE_FLAG_NONE },
   // Reported by Scott Call
@@ -148,6 +152,8 @@ static const LIBMTP_device_entry_t mtp_device_table[] = {
    */
   { "Toshiba Gigabeat MEGF-40", 0x0930, 0x0009, DEVICE_FLAG_NONE },
   { "Toshiba Gigabeat", 0x0930, 0x000c, DEVICE_FLAG_NONE },
+  // Reported by Rob Brown
+  { "Toshiba Gigabeat P10", 0x0930, 0x0011, DEVICE_FLAG_NONE },
 
   /*
    * Archos
@@ -224,16 +230,12 @@ static struct usb_device *probe_usb_bus_for_mtp_devices(void)
 	usb_close(devh);
 	continue;
       }
-
-      if (ret > 0) {
-	// printf("Device: VID %04x/PID %04x: responds to special descriptor call...\n",
-	//       dev->descriptor.idVendor,
-	//       dev->descriptor.idProduct);
-	// data_dump_ascii (stdout, buf, ret, 0);
-      }
-
+      
+      // It is atleast 10 bytes...
       if (!((buf[2] == 'M') && (buf[4]=='S') && (buf[6]=='F') && (buf[8]=='T'))) {
-	// printf("This is not a Microsoft MTP descriptor...\n");
+	printf("This is not a Microsoft MTP descriptor...\n");
+	printf("Device response to read device property 0xee:\n");
+	data_dump_ascii (stdout, buf, ret, 0);
 	usb_close(devh);
 	continue;
       }
@@ -244,13 +246,21 @@ static struct usb_device *probe_usb_bus_for_mtp_devices(void)
       if (ret == -1) {
 	//printf("Decice could not respond to control message 1.\n");
 	usb_close(devh);
-	// Return the device anyway.
-	return dev;
+	continue;
       }
       
-      if (ret > 0) {
-	//printf("Device response to control message 1:\n");
-	//data_dump_ascii (stdout, buf, ret, 0);
+      if (ret > 0x15) {
+	if ((buf[0x12] != 'M') || (buf[0x13] != 'T') || (buf[0x14] != 'P')) {
+	  printf("The device has a Microsoft device descriptor, but it's not MTP.\n");
+	  printf("This is not an MTP device. Presumable it is USB mass storage\n");
+	  printf("with some additional Janus (DRM) support.\n");
+	  printf("Device response to control message 1:\n");
+	  data_dump_ascii (stdout, buf, ret, 0);
+	  continue;
+	}
+      } else {
+	// Not MTP or broken
+	continue;
       }
 
       ret = usb_control_msg (devh, USB_ENDPOINT_IN|USB_RECIP_DEVICE|USB_TYPE_VENDOR, 
@@ -258,15 +268,20 @@ static struct usb_device *probe_usb_bus_for_mtp_devices(void)
       if (ret == -1) {
 	//printf("Device could not respond to control message 2.\n");
 	usb_close(devh);
-	// Return the device anyway.
+	// Return the device anyway, it said previously it was MTP, right?
 	return dev;
       }
       
-      if (ret > 0) {
-	//printf("Device response to control message 2:\n");
-	//data_dump_ascii (stdout, buf, ret, 0);
+      if (ret > 0x15) {
+	if ((buf[0x12] != 'M') || (buf[0x13] != 'T') || (buf[0x14] != 'P')) {
+	  printf("This device does not respond with MTP characteristics on\n");
+	  printf("the second device property read (0x05), but will be regarded\n");
+	  printf("as MTP anyway.\n");
+	  printf("Device response to control message 2:\n");
+	  data_dump_ascii (stdout, buf, ret, 0);
+	}
       }
-
+      
       usb_close(devh);
       // We can return the device here, it will be the first. 
       // If it was not MTP, the loop continues before it reaches this point.
@@ -341,7 +356,6 @@ ptp_read_func (unsigned char *bytes, unsigned int size, void *data, unsigned int
   int toread = 0;
   int result = 0;
   int curread = 0;
-	int i = 0;
   /* Split into small blocks. Too large blocks (>1x MB) would
    * timeout.
    */
@@ -669,17 +683,20 @@ next_step:
   }
   
   ret = ptp_opensession(params,1);
+  // printf("Session open (%d)...\n", ret);
   if (ret == PTP_RC_InvalidTransactionID) {
     params->transaction_id += 10;
     ret = ptp_opensession(params,1);
   }
   if (ret != PTP_RC_SessionAlreadyOpened && ret != PTP_RC_OK) {
+    printf("Could not open session! (Return code %d)\n  Try to reset the device.\n", ret);
     usb_release_interface(ptp_usb->handle,dev->config->interface->altsetting->bInterfaceNumber);
   }
 
   // It is actually permissible to call this before opening the session
   ret = ptp_getdeviceinfo(params, &deviceinfo);
   if (ret != PTP_RC_OK) {
+    printf("Could not get device info!\n");
     usb_release_interface(ptp_usb->handle,dev->config->interface->altsetting->bInterfaceNumber);
     return PTP_CD_RC_ERROR_CONNECTING;
   }
