@@ -358,11 +358,9 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 				sizeof(usbresp), params->data, &rlen);
 
 	if (ret!=PTP_RC_OK) {
-		printf("PTP_ERROR_IO\n");
 		ret = PTP_ERROR_IO;
 	} else
 	if (dtoh16(usbresp.type)!=PTP_USB_CONTAINER_RESPONSE) {
-		printf("PTP_ERROR_RESP_EXPECTED\n");
 		ret = PTP_ERROR_RESP_EXPECTED;
 	} else
 	if (dtoh16(usbresp.code)!=resp->Code) {
@@ -451,7 +449,7 @@ _ptp_transaction (PTPParams* params, PTPContainer* ptp,
 	}
 	/* get response */
 	CHECK_PTP_RC(params->getresp_func(params, ptp));
-	
+
 	return ptp->Code;
 }
 
@@ -681,6 +679,28 @@ ptp_getstorageinfo (PTPParams* params, uint32_t storageid,
 	return ret;
 }
 
+/**
+ * ptp_formatstore:
+ * params:	PTPParams*
+ *              storageid		- StorageID
+ *
+ * Formats the storage on the device.
+ *
+ * Return values: Some PTP_RC_* code.
+ **/
+uint16_t
+ptp_formatstore (PTPParams* params, uint32_t storageid)
+{
+	PTPContainer ptp;
+
+	PTP_CNT_INIT(ptp);
+	ptp.Code=PTP_OC_FormatStore;
+	ptp.Param1=storageid;
+	ptp.Param2=PTP_FST_Undefined;
+	ptp.Nparam=2;
+	return ptp_transaction(params, &ptp, PTP_DP_NODATA, 0, NULL, NULL);
+}
+
 /**
  * ptp_getobjecthandles:
  * params:	PTPParams*
@@ -1735,6 +1755,44 @@ ptp_canon_getfolderentries (PTPParams* params, uint32_t store, uint32_t p2,
 	return ret;
 }
 
+/**
+ * ptp_canon_lookup_object:
+ *
+ * This command looks up the specified object on the camera.
+ *
+ * Format is "A:\\PATH".
+ *
+ * The 'A' is the VolumeLabel from GetStorageInfo,
+ * my IXUS has "A" for the card and "V" for internal memory.
+ *  
+ * params:	PTPParams*
+ *      char* name - path name
+ *
+ * Return values: Some PTP_RC_* code.
+ *      uint32_t *oid - PTP object id.
+ *
+ **/
+uint16_t
+ptp_canon_lookup_object (PTPParams* params, char* name, uint32_t* objectid)
+{
+	uint16_t ret;
+	PTPContainer ptp;
+	unsigned char *data = NULL;
+	uint8_t len;
+
+	PTP_CNT_INIT (ptp);
+	ptp.Code=PTP_OC_CANON_LookupObject;
+	ptp.Nparam=0;
+	len=0;
+	data = malloc (2*(strlen(name)+1)+2);
+	memset (data, 0, 2*(strlen(name)+1)+2);
+	ptp_pack_string (params, name, data, 0, &len);
+	ret=ptp_transaction (params, &ptp, PTP_DP_SENDDATA, (len+1)*2+1, &data, NULL);
+	free (data);
+	*objectid = ptp.Param1;
+	return ret;
+}
+
 /**
  * ptp_canon_theme_download:
  *
@@ -1765,7 +1823,6 @@ ptp_canon_theme_download (PTPParams* params, uint32_t themenr,
 }
 
 
-
 uint16_t
 ptp_nikon_curve_download (PTPParams* params, unsigned char **data, unsigned int *size) {
 	PTPContainer ptp;
@@ -2577,8 +2634,8 @@ ptp_get_property_description(PTPParams* params, uint16_t dpc)
 		 N_("Auto White Balance Bias")},
 		{PTP_DPC_NIKON_WhiteBalanceTungstenBias,	/* 0xD018 */
 		 N_("Tungsten White Balance Bias")},
-		{PTP_DPC_NIKON_WhiteBalanceFlourescentBias,	/* 0xD019 */
-		 N_("Flourescent White Balance Bias")},
+		{PTP_DPC_NIKON_WhiteBalanceFluorescentBias,	/* 0xD019 */
+		 N_("Fluorescent White Balance Bias")},
 		{PTP_DPC_NIKON_WhiteBalanceDaylightBias,	/* 0xD01a */
 		 N_("Daylight White Balance Bias")},
 		{PTP_DPC_NIKON_WhiteBalanceFlashBias,		/* 0xD01b */
